@@ -117,7 +117,9 @@ class Reliquary:
     cogitation_errors = {}
 
     # We're pretty much picking up our configuration data
-    def __init__(self, input_settings, input_user=None, input_pass=None):
+    def __init__(
+        self, input_settings, input_user=None, input_pass=None, input_endpoint=None
+    ):
 
         # Load Settings from JSON
         try:
@@ -130,7 +132,11 @@ class Reliquary:
         try:
             # Let's start with basic global settings
             self.cogitation_verbosity = json_settings["settings"]["verbosity"]
-            self.cogitation_endpoint = json_settings["settings"]["endpoint"]
+            # Provide the option to specify the endpoint with the constructor, or pull it from the file
+            if input_endpoint and input_endpoint is str:
+                self.cogitation_endpoint = input_endpoint
+            else:
+                self.cogitation_endpoint = json_settings["settings"]["endpoint"]
             # TLS Tuning
             self.cogitation_certvalidation = json_settings["settings"]["tls"][
                 "validation"
@@ -267,14 +273,27 @@ class Reliquary:
         try:
             do_api_url = self.cogitation_endpoint + do_api_uri
             self.validate_url(do_api_url)
-            if not do_api_dryrun:
+            if not do_api_dryrun and type(do_api_payload) is dict:
                 do_api_r = requests.request(
                     do_api_verb,
                     url=do_api_url,
                     headers=self.cogitation_headers,
                     verify=self.cogitation_certvalidation,
                     auth=(self.cogitation_username, self.cogitation_password),
-                    json=do_api_payload,
+                    json=do_api_payload
+                )
+                # We'll be discarding the actual `Response` object after this, but we do want to get HTTP status for erro handling
+                response_code = do_api_r.status_code
+                do_api_r.raise_for_status()  # trigger an exception before trying to convert or read data. This should allow us to get good error info
+                return do_api_r.text  # if HTTP status is good, save response
+            elif not do_api_dryrun and type(do_api_payload) is str:
+                do_api_r = requests.request(
+                    do_api_verb,
+                    url=do_api_url,
+                    headers=self.cogitation_headers,
+                    verify=self.cogitation_certvalidation,
+                    auth=(self.cogitation_username, self.cogitation_password),
+                    data=do_api_payload
                 )
                 # We'll be discarding the actual `Response` object after this, but we do want to get HTTP status for erro handling
                 response_code = do_api_r.status_code
@@ -318,13 +337,25 @@ class Reliquary:
 
     # Executor of the API calls.
     # Takes optional arguments (variables, payload) depending on which method is used
-    def namshub(self, namshub_string, namshub_variables=False, namshub_dryrun=False):
+    def namshub(
+        self,
+        namshub_string,
+        namshub_variables=False,
+        namshub_dryrun=False,
+        namshub_payload=False,
+    ):
         # Sanitize the verb used to uppercase, fewer changes for mixup
         namshub_verb = self.get_play_verb(namshub_string).lower().upper()
         # URI is in JSON file
         namshub_resource = self.get_play_uri(namshub_string)
         # Grab the namshub payload from the json file, if it exists
-        namshub_payload = self.get_play_payload(namshub_string)
+        # If it already exists, you don't need to load it
+        if not namshub_payload:
+            namshub_payload = self.get_play_payload(namshub_string)
+        else:
+            # This payload would normally be a file, but we can process input too
+            # Let's fail-thru and try both methods
+            namshub_payload = self.get_json_file_or_string(namshub_payload)
         # Test to see if either variables or payloads are required
         # If they're required and not present, don't proceed
         if self.get_play_requiresvariables(namshub_string) and not namshub_variables:
@@ -333,7 +364,7 @@ class Reliquary:
             )
         if self.get_play_requiresbody(namshub_string) and not namshub_payload:
             sys.exit(
-                "Error: Payload required by play, but not provided! Specify as a JSON dictionary with `-b`"
+                "Error: Payload required by play, but not provided! Specify as a JSON dictionary with `-p`"
             )
 
         # Grab the variables, if it exists
@@ -341,11 +372,24 @@ class Reliquary:
         if namshub_variables:
             namshub_variables = self.get_json_file_or_string(namshub_variables)
             namshub_resource = self.apply_template(namshub_resource, namshub_variables)
-            if namshub_payload:
+            # Check to see if the payload is a string before templating the payload
+            if namshub_payload and namshub_payload is str:
                 for i in namshub_variables:
                     if i in namshub_payload:
                         namshub_payload[i] = namshub_variables[i]
 
+        # Make sure that the payload is a string before shippinng it to the API
+        if namshub_payload and namshub_payload not in [str, dict]:
+            try:
+                namshub_payload = json.dumps(namshub_payload, indent=4)
+            except Exception as e:
+                sys.exit(
+                    "Error processing API payload as "
+                    + str(type(namshub_payload))
+                    + " "
+                    + str(e)
+                )
+
         # Now that the transforms, testing, pre-processing are done, let's send to an API!
         if namshub_verb == "GET":
             return self.do_api_get(namshub_resource, do_api_dryrun=namshub_dryrun)
@@ -357,7 +401,7 @@ class Reliquary:
                 do_api_dryrun=namshub_dryrun,
             )
         elif namshub_verb == "DELETE":
-            return self.do_api_delete(namshub_resource)
+            return self.do_api_delete(namshub_resource, do_api_dryrun=namshub_dryrun)
         else:
             sys.exit("Unsupported API verb " + namshub_verb + "!")
 
@@ -458,12 +502,17 @@ class Reliquary:
             exit()
 
     def get_json_file_or_string(self, input):
+        # Recursive method to return a dictionary even if we're passed a dictionary
+        if type(input) is dict:
+            return input
+        # Try to load as a string
         try:
             return json.loads(input)
         except Exception as e:
             if self.cogitation_verbosity:
-                print(str(e))
+                print("Tried to load as a string, and found the following error: " + str(e))
             try:
+                # If it fails as a string, try to load as a file
                 return self.get_json_file(input)
             except Exception as e:
                 if self.cogitation_verbosity:
